@@ -24,7 +24,8 @@ require 'chef/mixin/path_sanity'
 require 'chef/log'
 require 'chef/rest'
 require 'chef/api_client'
-require 'chef/platform'
+require 'chef/api_client/registration'
+require 'chef/platform/query_helpers'
 require 'chef/node'
 require 'chef/role'
 require 'chef/file_cache'
@@ -35,7 +36,13 @@ require 'chef/cookbook/cookbook_collection'
 require 'chef/cookbook/file_vendor'
 require 'chef/cookbook/file_system_file_vendor'
 require 'chef/cookbook/remote_file_vendor'
+require 'chef/event_dispatch/dispatcher'
+require 'chef/formatters/base'
+require 'chef/formatters/doc'
+require 'chef/formatters/minimal'
 require 'chef/version'
+require 'chef/resource_reporter'
+require 'chef/run_lock'
 require 'ohai'
 require 'rbconfig'
 
@@ -96,18 +103,21 @@ class Chef
       self.class.run_start_notifications.each do |notification|
         notification.call(run_status)
       end
+      @events.run_started(run_status)
     end
 
     # Callback to fire notifications that the run completed successfully
     def run_completed_successfully
-      self.class.run_completed_successfully_notifications.each do |notification|
+      success_handlers = self.class.run_completed_successfully_notifications
+      success_handlers.each do |notification|
         notification.call(run_status)
       end
     end
 
     # Callback to fire notifications that the Chef run failed
     def run_failed
-      self.class.run_failed_notifications.each do |notification|
+      failure_handlers = self.class.run_failed_notifications
+      failure_handlers.each do |notification|
         notification.call(run_status)
       end
     end
@@ -120,8 +130,8 @@ class Chef
     #--
     # TODO: timh/cw: 5-19-2010: json_attribs should be moved to RunContext?
     attr_reader :json_attribs
-
     attr_reader :run_status
+    attr_reader :events
 
     # Creates a new Chef::Client.
     def initialize(json_attribs=nil, args={})
@@ -130,55 +140,92 @@ class Chef
       @run_status = nil
       @runner = nil
       @ohai = Ohai::System.new
+
+      event_handlers = configure_formatters
+      event_handlers += Array(Chef::Config[:event_handlers])
+
+      @events = EventDispatch::Dispatcher.new(*event_handlers)
       @override_runlist = args.delete(:override_runlist)
       runlist_override_sanity_check!
     end
 
-    # Do a full run for this Chef::Client.  Calls:
-    #
-    #  * run_ohai - Collect information about the system
-    #  * build_node - Get the last known state, merge with local changes
-    #  * register - If not in solo mode, make sure the server knows about this client
-    #  * sync_cookbooks - If not in solo mode, populate the local cache with the node's cookbooks
-    #  * converge - Bring this system up to date
-    #
-    # === Returns
-    # true:: Always returns true.
-    def run
-      run_context = nil
-
-      Chef::Log.info("*** Chef #{Chef::VERSION} ***")
-      enforce_path_sanity
-      run_ohai
-      register unless Chef::Config[:solo]
-      build_node
-
-      begin
-
-        run_status.start_clock
-        Chef::Log.info("Starting Chef Run for #{node.name}")
-        run_started
-
-        run_context = setup_run_context
-        converge(run_context)
-        save_updated_node
-
-        run_status.stop_clock
-        Chef::Log.info("Chef Run complete in #{run_status.elapsed_time} seconds")
-        run_completed_successfully
-        true
-      rescue Exception => e
-        run_status.stop_clock
-        run_status.exception = e
-        run_failed
-        Chef::Log.debug("Re-raising exception: #{e.class} - #{e.message}\n#{e.backtrace.join("\n  ")}")
-        raise
-      ensure
-        run_status = nil
+    def configure_formatters
+      formatters_for_run.map do |formatter_name, output_path|
+        if output_path.nil?
+          Chef::Formatters.new(formatter_name, STDOUT, STDERR)
+        else
+          io = File.open(output_path, "a+")
+          io.sync = true
+          Chef::Formatters.new(formatter_name, io, io)
+        end
       end
-      true
     end
 
+    def formatters_for_run
+      if Chef::Config.formatters.empty?
+        [default_formatter]
+      else
+        Chef::Config.formatters
+      end
+    end
+
+    def default_formatter
+      if (STDOUT.tty? && !Chef::Config[:force_logger]) || Chef::Config[:force_formatter]
+        [:doc]
+      else
+        [:null]
+      end
+    end
+
+    # Do a full run for this Chef::Client.  Calls:
+    # * do_run
+    #
+    # This provides a wrapper around #do_run allowing the
+    # run to be optionally forked.
+    # === Returns
+    # boolean:: Return value from #do_run. Should always returns true.
+    def run
+      # win32-process gem exposes some form of :fork for Process
+      # class. So we are seperately ensuring that the platform we're
+      # running on is not windows before forking.
+      if(Chef::Config[:client_fork] && Process.respond_to?(:fork) && !Chef::Platform.windows?)
+        Chef::Log.info "Forking chef instance to converge..."
+        pid = fork do
+          [:INT, :TERM].each {|s| trap(s, "EXIT") }
+          client_solo = Chef::Config[:solo] ? "chef-solo" : "chef-client"
+          $0 = "#{client_solo} worker: ppid=#{Process.ppid};start=#{Time.new.strftime("%R:%S")};"
+          begin
+            Chef::Log.debug "Forked instance now converging"
+            do_run
+          rescue Exception => e
+            Chef::Log.error(e.to_s)
+            exit 1
+          else
+            exit 0
+          end
+        end
+        Chef::Log.debug "Fork successful. Waiting for new chef pid: #{pid}"
+        result = Process.waitpid2(pid)
+        handle_child_exit(result)
+        Chef::Log.debug "Forked instance successfully reaped (pid: #{pid})"
+        true
+      else
+        do_run
+      end
+    end
+
+    def handle_child_exit(pid_and_status)
+      status = pid_and_status[1]
+      return true if status.success?
+      message = if status.signaled?
+        "Chef run process terminated by signal #{status.termsig} (#{Signal.list.invert[status.termsig]})"
+      else
+        "Chef run process exited unsuccessfully (exit code #{status.exitstatus})"
+      end
+      raise Exceptions::ChildConvergeError, message
+    end
+
+
 
     # Configures the Chef::Cookbook::FileVendor class to fetch file from the
     # server or disk as appropriate, creates the run context for this run, and
@@ -188,11 +235,15 @@ class Chef
     def setup_run_context
       if Chef::Config[:solo]
         Chef::Cookbook::FileVendor.on_create { |manifest| Chef::Cookbook::FileSystemFileVendor.new(manifest, Chef::Config[:cookbook_path]) }
-        run_context = Chef::RunContext.new(node, Chef::CookbookCollection.new(Chef::CookbookLoader.new(Chef::Config[:cookbook_path])))
+        cl = Chef::CookbookLoader.new(Chef::Config[:cookbook_path])
+        cl.load_cookbooks
+        cookbook_collection = Chef::CookbookCollection.new(cl)
+        run_context = Chef::RunContext.new(node, cookbook_collection, @events)
       else
         Chef::Cookbook::FileVendor.on_create { |manifest| Chef::Cookbook::RemoteFileVendor.new(manifest, rest) }
         cookbook_hash = sync_cookbooks
-        run_context = Chef::RunContext.new(node, Chef::CookbookCollection.new(cookbook_hash))
+        cookbook_collection = Chef::CookbookCollection.new(cookbook_hash)
+        run_context = Chef::RunContext.new(node, cookbook_collection, @events)
       end
       run_status.run_context = run_context
 
@@ -206,7 +257,7 @@ class Chef
         Chef::Log.debug("Saving the current state of node #{node_name}")
         if(@original_runlist)
           @node.run_list(*@original_runlist)
-          @node[:runlist_override_history] = {Time.now.to_i => @override_runlist.inspect}
+          @node.automatic_attrs[:runlist_override_history] = {Time.now.to_i => @override_runlist.inspect}
         end
         @node.save
       end
@@ -225,23 +276,21 @@ class Chef
         raise Chef::Exceptions::CannotDetermineNodeName, msg
       end
 
+      # node names > 90 bytes only work with authentication protocol >= 1.1
+      # see discussion in config.rb.
+      if name.bytesize > 90
+        Chef::Config[:authentication_protocol_version] = "1.1"
+      end
+
       name
     end
 
-    # Builds a new node object for this client.  Starts with querying for the FQDN of the current
-    # host (unless it is supplied), then merges in the facts from Ohai.
+    # Applies environment, external JSON attributes, and override run list to
+    # the node, Then expands the run_list.
     #
     # === Returns
-    # node<Chef::Node>:: Returns the created node object, also stored in @node
+    # node<Chef::Node>:: The modified @node object. @node is modified in place.
     def build_node
-      Chef::Log.debug("Building node object for #{node_name}")
-
-      if Chef::Config[:solo]
-        @node = Chef::Node.build(node_name)
-      else
-        @node = Chef::Node.find_or_create(node_name)
-      end
-
       # Allow user to override the environment of a node by specifying
       # a config parameter.
       if Chef::Config[:environment] && !Chef::Config[:environment].chop.empty?
@@ -263,11 +312,7 @@ class Chef
         Chef::Log.warn "Overridden Run List: [#{@node.run_list}]"
       end
 
-      if Chef::Config[:solo]
-        @run_list_expansion = @node.expand!('disk')
-      else
-        @run_list_expansion = @node.expand!('server')
-      end
+      @run_list_expansion = expand_run_list
 
       # @run_list_expansion is a RunListExpansion.
       #
@@ -282,23 +327,68 @@ class Chef
       Chef::Log.info("Run List is [#{@node.run_list}]")
       Chef::Log.info("Run List expands to [#{@expanded_run_list_with_versions.join(', ')}]")
 
-      @run_status = Chef::RunStatus.new(@node)
+      @run_status = Chef::RunStatus.new(@node, @events)
+
+      @events.node_load_completed(node, @expanded_run_list_with_versions, Chef::Config)
 
       @node
     end
 
+    # In client-server operation, loads the node state from the server. In
+    # chef-solo operation, builds a new node object.
+    def load_node
+      @events.node_load_start(node_name, Chef::Config)
+      Chef::Log.debug("Building node object for #{node_name}")
+
+      if Chef::Config[:solo]
+        @node = Chef::Node.build(node_name)
+      else
+        @node = Chef::Node.find_or_create(node_name)
+      end
+    rescue Exception => e
+      # TODO: wrap this exception so useful error info can be given to the
+      # user.
+      @events.node_load_failed(node_name, e, Chef::Config)
+      raise
+    end
+
+    def expand_run_list
+      if Chef::Config[:solo]
+        @node.expand!('disk')
+      else
+        @node.expand!('server')
+      end
+    rescue Exception => e
+      # TODO: wrap/munge exception with useful error output.
+      @events.run_list_expand_failed(node, e)
+      raise
+    end
+
     #
     # === Returns
     # rest<Chef::REST>:: returns Chef::REST connection object
     def register(client_name=node_name, config=Chef::Config)
-      if File.exists?(config[:client_key])
+      if !config[:client_key]
+        @events.skipping_registration(client_name, config)
+        Chef::Log.debug("Client key is unspecified - skipping registration")
+      elsif File.exists?(config[:client_key])
+        @events.skipping_registration(client_name, config)
         Chef::Log.debug("Client key #{config[:client_key]} is present - skipping registration")
       else
+        @events.registration_start(node_name, config)
         Chef::Log.info("Client key #{config[:client_key]} is not present - registering")
-        Chef::REST.new(config[:client_url], config[:validation_client_name], config[:validation_key]).register(client_name, config[:client_key])
+        Chef::ApiClient::Registration.new(node_name, config[:client_key]).run
+        @events.registration_completed
       end
       # We now have the client key, and should use it from now on.
-      self.rest = Chef::REST.new(config[:chef_server_url], client_name, config[:client_key])
+      @rest = Chef::REST.new(config[:chef_server_url], client_name, config[:client_key])
+      @resource_reporter = Chef::ResourceReporter.new(@rest)
+      @events.register(@resource_reporter)
+    rescue Exception => e
+      # TODO: munge exception so a semantic failure message can be given to the
+      # user
+      @events.registration_failed(node_name, e, config)
+      raise
     end
 
     # Sync_cookbooks eagerly loads all files except files and
@@ -310,9 +400,21 @@ class Chef
     # Hash:: The hash of cookbooks with download URLs as given by the server
     def sync_cookbooks
       Chef::Log.debug("Synchronizing cookbooks")
-      cookbook_hash = rest.post_rest("environments/#{@node.chef_environment}/cookbook_versions",
-                                     {:run_list => @expanded_run_list_with_versions})
-      Chef::CookbookVersion.sync_cookbooks(cookbook_hash)
+
+      begin
+        @events.cookbook_resolution_start(@expanded_run_list_with_versions)
+        cookbook_hash = rest.post_rest("environments/#{@node.chef_environment}/cookbook_versions",
+                                       {:run_list => @expanded_run_list_with_versions})
+      rescue Exception => e
+        # TODO: wrap/munge exception to provide helpful error output
+        @events.cookbook_resolution_failed(@expanded_run_list_with_versions, e)
+        raise
+      else
+        @events.cookbook_resolution_complete(cookbook_hash)
+      end
+
+      synchronizer = Chef::CookbookSynchronizer.new(cookbook_hash, @events)
+      synchronizer.sync_cookbooks
 
       # register the file cache path in the cookbook path so that CookbookLoader actually picks up the synced cookbooks
       Chef::Config[:cookbook_path] = File.join(Chef::Config[:file_cache_path], "cookbooks")
@@ -325,17 +427,112 @@ class Chef
     # === Returns
     # true:: Always returns true
     def converge(run_context)
+      @events.converge_start(run_context)
       Chef::Log.debug("Converging node #{node_name}")
       @runner = Chef::Runner.new(run_context)
       runner.converge
+      @events.converge_complete
       true
+    rescue Exception
+      # TODO: should this be a separate #converge_failed(exception) method?
+      @events.converge_complete
+      raise
+    end
+
+    def do_windows_admin_check
+      if Chef::Platform.windows?
+        Chef::Log.debug("Checking for administrator privileges....")
+
+        if !has_admin_privileges?
+          message = "chef-client doesn't have administrator privileges on node #{node_name}."
+          if Chef::Config[:fatal_windows_admin_check]
+            Chef::Log.fatal(message)
+            Chef::Log.fatal("fatal_windows_admin_check is set to TRUE.")
+            raise Chef::Exceptions::WindowsNotAdmin, message
+          else
+            Chef::Log.warn("#{message} This might cause unexpected resource failures.")
+          end
+        else
+          Chef::Log.debug("chef-client has administrator privileges on node #{node_name}.")
+        end
+      end
     end
 
     private
 
+    # Do a full run for this Chef::Client.  Calls:
+    #
+    #  * run_ohai - Collect information about the system
+    #  * build_node - Get the last known state, merge with local changes
+    #  * register - If not in solo mode, make sure the server knows about this client
+    #  * sync_cookbooks - If not in solo mode, populate the local cache with the node's cookbooks
+    #  * converge - Bring this system up to date
+    #
+    # === Returns
+    # true:: Always returns true.
+    def do_run
+      runlock = RunLock.new(Chef::Config.lockfile)
+      runlock.acquire
+      # don't add code that may fail before entering this section to be sure to release lock
+      begin
+        runlock.save_pid
+        run_context = nil
+        @events.run_start(Chef::VERSION)
+        Chef::Log.info("*** Chef #{Chef::VERSION} ***")
+        Chef::Log.info "Chef-client pid: #{Process.pid}"
+        enforce_path_sanity
+        run_ohai
+        @events.ohai_completed(node)
+        register unless Chef::Config[:solo]
+
+        load_node
+
+        build_node
+
+        run_status.start_clock
+        Chef::Log.info("Starting Chef Run for #{node.name}")
+        run_started
+
+        do_windows_admin_check
+
+        run_context = setup_run_context
+
+        converge(run_context)
+
+        save_updated_node
+
+        run_status.stop_clock
+        Chef::Log.info("Chef Run complete in #{run_status.elapsed_time} seconds")
+        run_completed_successfully
+        @events.run_completed(node)
+        true
+      rescue Exception => e
+        # CHEF-3336: Send the error first in case something goes wrong below and we don't know why
+        Chef::Log.debug("Re-raising exception: #{e.class} - #{e.message}\n#{e.backtrace.join("\n  ")}")
+        # If we failed really early, we may not have a run_status yet. Too early for these to be of much use.
+        if run_status
+          run_status.stop_clock
+          run_status.exception = e
+          run_failed
+        end
+        Chef::Application.debug_stacktrace(e)
+        @events.run_failed(e)
+        raise
+      ensure
+        @run_status = nil
+        run_context = nil
+        runlock.release
+        GC.start
+      end
+      true
+    end
+
     # Ensures runlist override contains RunListItem instances
     def runlist_override_sanity_check!
-      @override_runlist = @override_runlist.split(',') if @override_runlist.is_a?(String)
+      # Convert to array and remove whitespace
+      if @override_runlist.is_a?(String)
+        @override_runlist = @override_runlist.split(',').map { |e| e.strip }
+      end
       @override_runlist = [@override_runlist].flatten.compact
       @override_runlist.map! do |item|
         if(item.is_a?(Chef::RunList::RunListItem))
@@ -374,10 +571,18 @@ class Chef
       end
 
     end
+
+    def has_admin_privileges?
+      require 'chef/win32/security'
+
+      Chef::ReservedNames::Win32::Security.has_admin_privileges?
+    end
+
   end
 end
 
 # HACK cannot load this first, but it must be loaded.
 require 'chef/cookbook_loader'
 require 'chef/cookbook_version'
+require 'chef/cookbook/synchronizer'
 
