@@ -21,7 +21,6 @@
 require 'chef/mash'
 require 'chef/mixin/from_file'
 require 'chef/mixin/params_validate'
-require 'chef/mixin/check_helper'
 require 'chef/log'
 require 'chef/version_class'
 require 'chef/version_constraint'
@@ -64,7 +63,6 @@ class Chef
                              :provides    => PROVIDING,
                              :replaces    => REPLACING }
 
-      include Chef::Mixin::CheckHelper
       include Chef::Mixin::ParamsValidate
       include Chef::Mixin::FromFile
 
@@ -112,8 +110,8 @@ class Chef
         @version = Version.new "0.0.0"
         if cookbook
           @recipes = cookbook.fully_qualified_recipe_names.inject({}) do |r, e|
-            e = self.name if e =~ /::default$/
-            r[e] = ""
+            e = self.name.to_s if e =~ /::default$/
+            r[e] ||= ""
             self.provides e
             r
           end
