@@ -3,11 +3,11 @@ require 'set'
 require 'rest_client'
 require 'chef/exceptions'
 require 'chef/knife/cookbook_metadata'
-require 'chef/checksum_cache'
-require 'chef/sandbox'
+require 'chef/digester'
 require 'chef/cookbook_version'
 require 'chef/cookbook/syntax_check'
 require 'chef/cookbook/file_system_file_vendor'
+require 'chef/sandbox'
 
 class Chef
   class CookbookUploader
@@ -29,14 +29,15 @@ class Chef
       end
     end
 
-    attr_reader :cookbook
+    attr_reader :cookbooks
     attr_reader :path
     attr_reader :opts
     attr_reader :rest
 
     # Creates a new CookbookUploader.
     # ===Arguments:
-    # * cookbook::: A Chef::CookbookVersion describing the cookbook to be uploaded
+    # * cookbooks::: A Chef::CookbookVersion or array of them describing the
+    #                cookbook(s) to be uploaded
     # * path::: A String or Array of Strings representing the base paths to the
     #           cookbook repositories.
     # * opts::: (optional) An options Hash
@@ -48,19 +49,24 @@ class Chef
     # * :rest   A Chef::REST object that you have configured the way you like it.
     #           If you don't provide this, one will be created using the values
     #           in Chef::Config.
-    def initialize(cookbook, path, opts={})
-      @cookbook, @path, @opts = cookbook, path, opts
+    def initialize(cookbooks, path, opts={})
+      @path, @opts = path, opts
+      @cookbooks = Array(cookbooks)
       @rest = opts[:rest] || Chef::REST.new(Chef::Config[:chef_server_url])
     end
 
-    def upload_cookbook
+    def upload_cookbooks
       Thread.abort_on_exception = true
-      Chef::Log.info("Saving #{cookbook.name}")
 
       # Syntax Check
-      validate_cookbook
+      validate_cookbooks
       # generate checksums of cookbook files and create a sandbox
-      checksum_files = cookbook.checksums
+      checksum_files = {}
+      cookbooks.each do |cb|
+        Chef::Log.info("Saving #{cb.name}")
+        checksum_files.merge!(cb.checksums)
+      end
+
       checksums = checksum_files.inject({}){|memo,elt| memo[elt.first]=nil ; memo}
       new_sandbox = rest.post_rest("sandboxes", { :checksums => checksums })
 
@@ -102,8 +108,19 @@ class Chef
       end
 
       # files are uploaded, so save the manifest
-      save_url = opts[:force] ? cookbook.force_save_url : cookbook.save_url
-      rest.put_rest(save_url, cookbook)
+      cookbooks.each do |cb|
+        save_url = opts[:force] ? cb.force_save_url : cb.save_url
+        begin
+          rest.put_rest(save_url, cb)
+        rescue Net::HTTPServerException => e
+          case e.response.code
+          when "409"
+            raise Chef::Exceptions::CookbookFrozen, "Version #{cb.version} of cookbook #{cb.name} is frozen. Use --force to override."
+          else
+            raise
+          end
+        end
+      end
 
       Chef::Log.info("Upload complete!")
     end
@@ -120,15 +137,17 @@ class Chef
         timestamp = Time.now.utc.iso8601
         file_contents = File.open(file, "rb") {|f| f.read}
         # TODO - 5/28/2010, cw: make signing and sending the request streaming
-        sign_obj = Mixlib::Authentication::SignedHeaderAuth.signing_object(
-                                                                           :http_method => :put,
-                                                                           :path        => URI.parse(url).path,
-                                                                           :body        => file_contents,
-                                                                           :timestamp   => timestamp,
-                                                                           :user_id     => rest.client_name
-                                                                           )
         headers = { 'content-type' => 'application/x-binary', 'content-md5' => checksum64, :accept => 'application/json' }
-        headers.merge!(sign_obj.sign(OpenSSL::PKey::RSA.new(rest.signing_key)))
+        if rest.signing_key
+          sign_obj = Mixlib::Authentication::SignedHeaderAuth.signing_object(
+                                                                             :http_method => :put,
+                                                                             :path        => URI.parse(url).path,
+                                                                             :body        => file_contents,
+                                                                             :timestamp   => timestamp,
+                                                                             :user_id     => rest.client_name
+                                                                             )
+          headers.merge!(sign_obj.sign(OpenSSL::PKey::RSA.new(rest.signing_key)))
+        end
 
         begin
           RestClient::Resource.new(url, :headers=>headers, :timeout=>1800, :open_timeout=>1800).put(file_contents)
@@ -140,14 +159,16 @@ class Chef
       end
     end
 
-    def validate_cookbook
-      syntax_checker = Chef::Cookbook::SyntaxCheck.for_cookbook(cookbook.name, @user_cookbook_path)
-      Chef::Log.info("Validating ruby files")
-      exit(1) unless syntax_checker.validate_ruby_files
-      Chef::Log.info("Validating templates")
-      exit(1) unless syntax_checker.validate_templates
-      Chef::Log.info("Syntax OK")
-      true
+    def validate_cookbooks
+      cookbooks.each do |cb|
+        syntax_checker = Chef::Cookbook::SyntaxCheck.for_cookbook(cb.name, @user_cookbook_path)
+        Chef::Log.info("Validating ruby files")
+        exit(1) unless syntax_checker.validate_ruby_files
+        Chef::Log.info("Validating templates")
+        exit(1) unless syntax_checker.validate_templates
+        Chef::Log.info("Syntax OK")
+        true
+      end
     end
 
   end
