@@ -20,6 +20,7 @@
 require 'forwardable'
 require 'chef/version'
 require 'mixlib/cli'
+require 'chef/config_fetcher'
 require 'chef/mixin/convert_to_class_name'
 require 'chef/mixin/path_sanity'
 require 'chef/knife/core/subcommand_loader'
@@ -57,6 +58,11 @@ class Chef
     attr_accessor :name_args
     attr_accessor :ui
 
+    # Configure mixlib-cli to always separate defaults from user-supplied CLI options
+    def self.use_separate_defaults?
+      true
+    end
+
     def self.ui
       @ui ||= Chef::Knife::UI.new(STDOUT, STDERR, STDIN, {})
     end
@@ -145,7 +151,7 @@ class Chef
       commands_to_show.sort.each do |category, commands|
         next if category =~ /deprecated/i
         msg "** #{category.upcase} COMMANDS **"
-        commands.each do |command|
+        commands.sort.each do |command|
           msg subcommands[command].banner if subcommands[command]
         end
         msg
@@ -196,12 +202,18 @@ class Chef
       subcommand_class || subcommand_not_found!(args)
     end
 
+    def self.dependency_loaders
+      @dependency_loaders ||= []
+    end
+
     def self.deps(&block)
-      @dependency_loader = block
+      dependency_loaders << block
     end
 
     def self.load_deps
-      @dependency_loader && @dependency_loader.call
+      dependency_loaders.each do |dep_loader|
+        dep_loader.call
+      end
     end
 
     private
@@ -226,13 +238,22 @@ class Chef
       exit 10
     end
 
-    @@chef_config_dir = nil
+    def self.working_directory
+      ENV['PWD'] || Dir.pwd
+    end
+
+    def self.reset_config_path!
+      @@chef_config_dir = nil
+    end
+
+    reset_config_path!
+
 
     # search upward from current_dir until .chef directory is found
     def self.chef_config_dir
       if @@chef_config_dir.nil? # share this with subclasses
         @@chef_config_dir = false
-        full_path = Dir.pwd.split(File::SEPARATOR)
+        full_path = working_directory.split(File::SEPARATOR)
         (full_path.length - 1).downto(0) do |i|
           candidate_directory = File.join(full_path[0..i] + [".chef" ])
           if File.exist?(candidate_directory) && File.directory?(candidate_directory)
@@ -269,6 +290,10 @@ class Chef
         msg opt_parser
         exit 1
       end
+
+      # copy Mixlib::CLI over so that it cab be configured in knife.rb
+      # config file
+      Chef::Config[:verbosity] = config[:verbosity]
     end
 
     def parse_options(args)
@@ -279,49 +304,74 @@ class Chef
       exit(1)
     end
 
-    def configure_chef
-      unless config[:config_file]
-        candidate_configs = []
+    # Returns a subset of the Chef::Config[:knife] Hash that is relevant to the
+    # currently executing knife command. This is used by #configure_chef to
+    # apply settings from knife.rb to the +config+ hash.
+    def config_file_settings
+      config_file_settings = {}
+      self.class.options.keys.each do |key|
+        config_file_settings[key] = Chef::Config[:knife][key] if Chef::Config[:knife].has_key?(key)
+      end
+      config_file_settings
+    end
 
-        # Look for $KNIFE_HOME/knife.rb (allow multiple knives config on same machine)
-        if ENV['KNIFE_HOME']
-          candidate_configs << File.join(ENV['KNIFE_HOME'], 'knife.rb')
-        end
-        # Look for $PWD/knife.rb
-        if Dir.pwd
-          candidate_configs << File.join(Dir.pwd, 'knife.rb')
-        end
-        # Look for $UPWARD/.chef/knife.rb
-        if self.class.chef_config_dir
-          candidate_configs << File.join(self.class.chef_config_dir, 'knife.rb')
-        end
-        # Look for $HOME/.chef/knife.rb
-        if ENV['HOME']
-          candidate_configs << File.join(ENV['HOME'], '.chef', 'knife.rb')
-        end
+    def self.config_fetcher(candidate_config)
+      Chef::ConfigFetcher.new(candidate_config, Chef::Config.config_file_jail)
+    end
 
-        candidate_configs.each do | candidate_config |
-          candidate_config = File.expand_path(candidate_config)
-          if File.exist?(candidate_config)
-            config[:config_file] = candidate_config
-            break
-          end
-        end
+    def self.locate_config_file
+      candidate_configs = []
+
+      # Look for $KNIFE_HOME/knife.rb (allow multiple knives config on same machine)
+      if ENV['KNIFE_HOME']
+        candidate_configs << File.join(ENV['KNIFE_HOME'], 'knife.rb')
+      end
+      # Look for $PWD/knife.rb
+      if Dir.pwd
+        candidate_configs << File.join(Dir.pwd, 'knife.rb')
+      end
+      # Look for $UPWARD/.chef/knife.rb
+      if chef_config_dir
+        candidate_configs << File.join(chef_config_dir, 'knife.rb')
+      end
+      # Look for $HOME/.chef/knife.rb
+      if ENV['HOME']
+        candidate_configs << File.join(ENV['HOME'], '.chef', 'knife.rb')
       end
 
-      # Don't try to load a knife.rb if it doesn't exist.
-      if config[:config_file]
-        read_config_file(config[:config_file])
-      else
-        # ...but do log a message if no config was found.
-        Chef::Config[:color] = config[:color]
-        ui.warn("No knife configuration file found")
+      candidate_configs.each do | candidate_config |
+        fetcher = config_fetcher(candidate_config)
+        if !fetcher.config_missing?
+          return candidate_config
+        end
       end
+      return nil
+    end
 
+    # Apply Config in this order:
+    # defaults from mixlib-cli
+    # settings from config file, via Chef::Config[:knife]
+    # config from command line
+    def merge_configs
+      # Apply config file settings on top of mixlib-cli defaults
+      combined_config = default_config.merge(config_file_settings)
+      # Apply user-supplied options on top of the above combination
+      combined_config = combined_config.merge(config)
+      # replace the config hash from mixlib-cli with our own.
+      # Need to use the mutate-in-place #replace method instead of assigning to
+      # the instance variable because other code may have a reference to the
+      # original config hash object.
+      config.replace(combined_config)
+    end
+
+    # Catch-all method that does any massaging needed for various config
+    # components, such as expanding file paths and converting verbosity level
+    # into log level.
+    def apply_computed_config
       Chef::Config[:color] = config[:color]
 
-      case config[:verbosity]
-      when 0
+      case Chef::Config[:verbosity]
+      when 0, nil
         Chef::Config[:log_level] = :error
       when 1
         Chef::Config[:log_level] = :info
@@ -334,6 +384,12 @@ class Chef
       Chef::Config[:chef_server_url]   = config[:chef_server_url] if config[:chef_server_url]
       Chef::Config[:environment]       = config[:environment]     if config[:environment]
 
+      Chef::Config.local_mode = config[:local_mode] if config.has_key?(:local_mode)
+      if Chef::Config.local_mode && !Chef::Config.has_key?(:cookbook_path) && !Chef::Config.has_key?(:chef_repo_path)
+        Chef::Config.chef_repo_path = Chef::Config.find_chef_repo_path(Dir.pwd)
+      end
+      Chef::Config.chef_zero.port = config[:chef_zero_port] if config[:chef_zero_port]
+
       # Expand a relative path from the config directory. Config from command
       # line should already be expanded, and absolute paths will be unchanged.
       if Chef::Config[:client_key] && config[:config_file]
@@ -344,31 +400,56 @@ class Chef
       Chef::Log.init(Chef::Config[:log_location])
       Chef::Log.level(Chef::Config[:log_level] || :error)
 
-      Chef::Log.debug("Using configuration from #{config[:config_file]}")
-
-      if Chef::Config[:node_name].nil?
-        #raise ArgumentError, "No user specified, pass via -u or specifiy 'node_name' in #{config[:config_file] ? config[:config_file] : "~/.chef/knife.rb"}"
+      if Chef::Config[:node_name] && Chef::Config[:node_name].bytesize > 90
+        # node names > 90 bytes only work with authentication protocol >= 1.1
+        # see discussion in config.rb.
+        Chef::Config[:authentication_protocol_version] = "1.1"
       end
     end
 
-    def read_config_file(file)
-      Chef::Config.from_file(file)
+    def configure_chef
+      if !config[:config_file]
+        located_config_file = self.class.locate_config_file
+        config[:config_file] = located_config_file if located_config_file
+      end
+
+      # Don't try to load a knife.rb if it wasn't specified.
+      if config[:config_file]
+        fetcher = Chef::ConfigFetcher.new(config[:config_file], Chef::Config.config_file_jail)
+        if fetcher.config_missing?
+          ui.error("Specified config file #{config[:config_file]} does not exist#{Chef::Config.config_file_jail ? " or is not under config file jail #{Chef::Config.config_file_jail}" : ""}!")
+          exit 1
+        end
+        Chef::Log.debug("Using configuration from #{config[:config_file]}")
+        read_config(fetcher.read_config, config[:config_file])
+      else
+        # ...but do log a message if no config was found.
+        Chef::Config[:color] = config[:color]
+        ui.warn("No knife configuration file found")
+      end
+
+      merge_configs
+      apply_computed_config
+    end
+
+    def read_config(config_content, config_file_path)
+      Chef::Config.from_string(config_content, config_file_path)
     rescue SyntaxError => e
-      ui.error "You have invalid ruby syntax in your config file #{file}"
+      ui.error "You have invalid ruby syntax in your config file #{config_file_path}"
       ui.info(ui.color(e.message, :red))
-      if file_line = e.message[/#{Regexp.escape(file)}:[\d]+/]
+      if file_line = e.message[/#{Regexp.escape(config_file_path)}:[\d]+/]
         line = file_line[/:([\d]+)$/, 1].to_i
-        highlight_config_error(file, line)
+        highlight_config_error(config_file_path, line)
       end
       exit 1
     rescue Exception => e
-      ui.error "You have an error in your config file #{file}"
+      ui.error "You have an error in your config file #{config_file_path}"
       ui.info "#{e.class.name}: #{e.message}"
-      filtered_trace = e.backtrace.grep(/#{Regexp.escape(file)}/)
+      filtered_trace = e.backtrace.grep(/#{Regexp.escape(config_file_path)}/)
       filtered_trace.each {|line| ui.msg("  " + ui.color(line, :red))}
       if !filtered_trace.empty?
-        line_nr = filtered_trace.first[/#{Regexp.escape(file)}:([\d]+)/, 1]
-        highlight_config_error(file, line_nr.to_i)
+        line_nr = filtered_trace.first[/#{Regexp.escape(config_file_path)}:([\d]+)/, 1]
+        highlight_config_error(config_file_path, line_nr.to_i)
       end
 
       exit 1
@@ -394,14 +475,19 @@ class Chef
       stdout.puts("USAGE: " + self.opt_parser.to_s)
     end
 
-    def run_with_pretty_exceptions
+    def run_with_pretty_exceptions(raise_exception = false)
       unless self.respond_to?(:run)
         ui.error "You need to add a #run method to your knife command before you can use it"
       end
       enforce_path_sanity
-      run
+      Chef::Application.setup_server_connectivity
+      begin
+        run
+      ensure
+        Chef::Application.destroy_server_connectivity
+      end
     rescue Exception => e
-      raise if config[:verbosity] == 2
+      raise if raise_exception || Chef::Config[:verbosity] == 2
       humanize_exception(e)
       exit 100
     end
@@ -423,6 +509,9 @@ class Chef
       when Chef::Exceptions::PrivateKeyMissing
         ui.error "Your private key could not be loaded from #{api_key}"
         ui.info  "Check your configuration file and ensure that your private key is readable"
+      when Chef::Exceptions::InvalidRedirect
+        ui.error "Invalid Redirect: #{e.message}"
+        ui.info  "Change your server location in knife.rb to the server's FQDN to avoid unwanted redirections."
       else
         ui.error "#{e.class.name}: #{e.message}"
       end
