@@ -25,9 +25,12 @@ class Chef
       deps do
         require 'net/ssh'
         require 'net/ssh/multi'
+        require 'chef/monkey_patches/net-ssh-multi'
         require 'readline'
+        require 'chef/exceptions'
         require 'chef/search/query'
         require 'chef/mixin/shell_out'
+        require 'mixlib/shellout'
       end
 
       include Chef::Mixin::ShellOut
@@ -46,8 +49,8 @@ class Chef
       option :attribute,
         :short => "-a ATTR",
         :long => "--attribute ATTR",
-        :description => "The attribute to use for opening the connection - default is fqdn",
-        :proc => Proc.new { |key| Chef::Config[:knife][:ssh_attribute] = key }
+        :description => "The attribute to use for opening the connection - default depends on the context",
+        :proc => Proc.new { |key| Chef::Config[:knife][:ssh_attribute] = key.strip }
 
       option :manual,
         :short => "-m",
@@ -70,14 +73,19 @@ class Chef
         :short => "-p PORT",
         :long => "--ssh-port PORT",
         :description => "The ssh port",
-        :default => "22",
-        :proc => Proc.new { |key| Chef::Config[:knife][:ssh_port] = key }
+        :proc => Proc.new { |key| Chef::Config[:knife][:ssh_port] = key.strip }
 
       option :ssh_gateway,
         :short => "-G GATEWAY",
         :long => "--ssh-gateway GATEWAY",
         :description => "The ssh gateway",
-        :proc => Proc.new { |key| Chef::Config[:knife][:ssh_gatewa] = key }
+        :proc => Proc.new { |key| Chef::Config[:knife][:ssh_gateway] = key.strip }
+
+      option :forward_agent,
+        :short => "-A",
+        :long => "--forward-agent",
+        :description => "Enable SSH agent forwarding",
+        :boolean => true
 
       option :identity_file,
         :short => "-i IDENTITY_FILE",
@@ -113,6 +121,22 @@ class Chef
         @session ||= Net::SSH::Multi.start(:concurrent_connections => config[:concurrency], :on_error => ssh_error_handler)
       end
 
+      def configure_gateway
+        config[:ssh_gateway] ||= Chef::Config[:knife][:ssh_gateway]
+        if config[:ssh_gateway]
+          gw_host, gw_user = config[:ssh_gateway].split('@').reverse
+          gw_host, gw_port = gw_host.split(':')
+          gw_opts = gw_port ? { :port => gw_port } : {}
+
+          session.via(gw_host, gw_user || config[:ssh_user], gw_opts)
+        end
+      rescue Net::SSH::AuthenticationFailed
+        user = gw_user || config[:ssh_user]
+        prompt = "Enter the password for #{user}@#{gw_host}: "
+        gw_opts.merge!(:password => prompt_for_password(prompt))
+        session.via(gw_host, user, gw_opts)
+      end
+
       def configure_session
         list = case config[:manual]
                when true
@@ -122,34 +146,51 @@ class Chef
                  q = Chef::Search::Query.new
                  @action_nodes = q.search(:node, @name_args[0])[0]
                  @action_nodes.each do |item|
-                   i = format_for_display(item)[config[:attribute]]
-                   r.push(i) unless i.nil?
+                   # we should skip the loop to next iteration if the item returned by the search is nil
+                   next if item.nil?
+                   # if a command line attribute was not passed, and we have a cloud public_hostname, use that.
+                   # see #configure_attribute for the source of config[:attribute] and config[:override_attribute]
+                   if !config[:override_attribute] && item[:cloud] and item[:cloud][:public_hostname]
+                     i = item[:cloud][:public_hostname]
+                   elsif config[:override_attribute]
+                     i = extract_nested_value(item, config[:override_attribute])
+                   else
+                     i = extract_nested_value(item, config[:attribute])
+                   end
+                   # next if we couldn't find the specified attribute in the returned node object
+                   next if i.nil?
+                   r.push(i)
                  end
                  r
                end
-        (ui.fatal("No nodes returned from search!"); exit 10) if list.length == 0
+        if list.length == 0
+          if @action_nodes.length == 0
+            ui.fatal("No nodes returned from search!")
+          else
+            ui.fatal("#{@action_nodes.length} #{@action_nodes.length > 1 ? "nodes":"node"} found, " +
+                     "but does not have the required attribute to establish the connection. " +
+                     "Try setting another attribute to open the connection using --attribute.")
+          end
+          exit 10
+        end
         session_from_list(list)
       end
 
       def session_from_list(list)
-        config[:ssh_gateway] ||= Chef::Config[:knife][:ssh_gateway]
-        if config[:ssh_gateway]
-          gw_host, gw_user = config[:ssh_gateway].split('@').reverse
-          gw_host, gw_port = gw_host.split(':')
-          gw_opts = gw_port ? { :port => gw_port } : {}
-
-          session.via(gw_host, gw_user || config[:ssh_user], gw_opts)
-        end
-
         list.each do |item|
           Chef::Log.debug("Adding #{item}")
-
-          hostspec = config[:ssh_user] ? "#{config[:ssh_user]}@#{item}" : item
           session_opts = {}
+
+          ssh_config = Net::SSH.configuration_for(item)
+
+          # Chef::Config[:knife][:ssh_user] is parsed in #configure_user and written to config[:ssh_user]
+          user = config[:ssh_user] || ssh_config[:user]
+          hostspec = user ? "#{user}@#{item}" : item
           session_opts[:keys] = File.expand_path(config[:identity_file]) if config[:identity_file]
           session_opts[:keys_only] = true if config[:identity_file]
           session_opts[:password] = config[:ssh_password] if config[:ssh_password]
-          session_opts[:port] = Chef::Config[:knife][:ssh_port] || config[:ssh_port]
+          session_opts[:forward_agent] = config[:forward_agent]
+          session_opts[:port] = config[:ssh_port] || Chef::Config[:knife][:ssh_port] || ssh_config[:port]
           session_opts[:logger] = Chef::Log.logger if Chef::Log.level == :debug
 
           if !config[:host_key_verify]
@@ -170,18 +211,33 @@ class Chef
       end
 
       def print_data(host, data)
-        if data =~ /\n/
-          data.split(/\n/).each { |d| print_data(host, d) }
+        @buffers ||= {}
+        if leftover = @buffers[host]
+          @buffers[host] = nil
+          print_data(host, leftover + data)
         else
-          padding = @longest - host.length
-          str = ui.color(host, :cyan) + (" " * (padding + 1)) + data
-          ui.msg(str)
+          if newline_index = data.index("\n")
+            line = data.slice!(0...newline_index)
+            data.slice!(0)
+            print_line(host, line)
+            print_data(host, data)
+          else
+            @buffers[host] = data
+          end
         end
       end
 
+      def print_line(host, data)
+        padding = @longest - host.length
+        str = ui.color(host, :cyan) + (" " * (padding + 1)) + data
+        ui.msg(str)
+      end
+
       def ssh_command(command, subsession=nil)
+        exit_status = 0
         subsession ||= session
         command = fixup_sudo(command)
+        command.force_encoding('binary') if command.respond_to?(:force_encoding)
         subsession.open_channel do |ch|
           ch.request_pty
           ch.exec command do |ch, success|
@@ -189,16 +245,25 @@ class Chef
             ch.on_data do |ichannel, data|
               print_data(ichannel[:host], data)
               if data =~ /^knife sudo password: /
+                print_data(ichannel[:host], "\n")
                 ichannel.send_data("#{get_password}\n")
               end
             end
+            ch.on_request "exit-status" do |ichannel, data|
+              exit_status = [exit_status, data.read_long].max
+            end
           end
         end
         session.loop
+        exit_status
       end
 
       def get_password
-        @password ||= ui.ask("Enter your password: ") { |q| q.echo = false }
+        @password ||= prompt_for_password
+      end
+
+      def prompt_for_password(prompt = "Enter your password: ")
+        ui.ask(prompt) { |q| q.echo = false }
       end
 
       # Present the prompt and read a single line from the console. It also
@@ -327,17 +392,34 @@ class Chef
       end
 
       def configure_attribute
+        # Setting 'knife[:ssh_attribute] = "foo"' in knife.rb => Chef::Config[:knife][:ssh_attribute] == 'foo'
+        # Running 'knife ssh -a foo' => both Chef::Config[:knife][:ssh_attribute] && config[:attribute] == foo
+        # Thus we can differentiate between a config file value and a command line override at this point by checking config[:attribute]
+        # We can tell here if fqdn was passed from the command line, rather than being the default, by checking config[:attribute]
+        # However, after here, we cannot tell these things, so we must preserve config[:attribute]
+        config[:override_attribute] = config[:attribute] || Chef::Config[:knife][:ssh_attribute]
         config[:attribute] = (Chef::Config[:knife][:ssh_attribute] ||
                               config[:attribute] ||
                               "fqdn").strip
       end
 
-      def csshx
-        csshx_cmd = "csshX"
-        session.servers_for.each do |server|
-          csshx_cmd << " #{server.user ? "#{server.user}@#{server.host}" : server.host}"
+      def cssh
+        cssh_cmd = nil
+        %w[csshX cssh].each do |cmd|
+          begin
+            # Unix and Mac only
+            cssh_cmd = shell_out!("which #{cmd}").stdout.strip
+            break
+          rescue Mixlib::ShellOut::ShellCommandFailed
+          end
         end
-        exec(csshx_cmd)
+        raise Chef::Exceptions::Exec, "no command found for cssh" unless cssh_cmd
+
+        session.servers_for.each do |server|
+          cssh_cmd << " #{server.user ? "#{server.user}@#{server.host}" : server.host}"
+        end
+        Chef::Log.debug("starting cssh session with command: #{cssh_cmd}")
+        exec(cssh_cmd)
       end
 
       def get_stripped_unfrozen_value(value)
@@ -351,10 +433,14 @@ class Chef
       end
 
       def configure_identity_file
-        config[:identity_file] = get_stripped_unfrozen_value(config[:identity_file] || 
+        config[:identity_file] = get_stripped_unfrozen_value(config[:identity_file] ||
                              Chef::Config[:knife][:ssh_identity_file])
       end
 
+      def extract_nested_value(data_structure, path_spec)
+        ui.presenter.extract_nested_value(data_structure, path_spec)
+      end
+
       def run
         extend Chef::Mixin::Command
 
@@ -363,8 +449,10 @@ class Chef
         configure_attribute
         configure_user
         configure_identity_file
+        configure_gateway
         configure_session
 
+        exit_status =
         case @name_args[1]
         when "interactive"
           interactive
@@ -374,16 +462,24 @@ class Chef
           tmux
         when "macterm"
           macterm
+        when "cssh"
+          cssh
         when "csshx"
-          csshx
+          Chef::Log.warn("knife ssh csshx will be deprecated in a future release")
+          Chef::Log.warn("please use knife ssh cssh instead")
+          cssh
         else
           ssh_command(@name_args[1..-1].join(" "))
         end
 
         session.close
+        if exit_status != 0
+          exit exit_status
+        else
+          exit_status
+        end
       end
 
     end
   end
 end
-
