@@ -6,9 +6,9 @@
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
-# 
+#
 #     http://www.apache.org/licenses/LICENSE-2.0
-# 
+#
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS,
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -19,18 +19,18 @@
 class Chef
   class Provider
     class Breakpoint < Chef::Provider
-      
+
       def load_current_resource
       end
-      
+
       def action_break
-        if defined?(Shef) && Shef.running?
+        if defined?(Shell) && Shell.running?
           run_context.resource_collection.iterator.pause
           @new_resource.updated_by_last_action(true)
           run_context.resource_collection.iterator
         end
       end
-      
+
     end
   end
 end
