@@ -25,7 +25,7 @@ class Chef
     class Cron < Chef::Provider
       include Chef::Mixin::Command
 
-      CRON_PATTERN = /\A([-0-9*,\/]+)\s([-0-9*,\/]+)\s([-0-9*,\/]+)\s([-0-9*,\/]+)\s([-0-9*,\/]+)\s(.*)/
+      CRON_PATTERN = /\A([-0-9*,\/]+)\s([-0-9*,\/]+)\s([-0-9*,\/]+)\s([-0-9*,\/]+|[a-zA-Z]{3})\s([-0-9*,\/]+|[a-zA-Z]{3})\s(.*)/
       ENV_PATTERN = /\A(\S+)=(\S*)/
 
       CRON_ATTRIBUTES = [:minute, :hour, :day, :month, :weekday, :command, :mailto, :path, :shell, :home, :environment]
@@ -37,10 +37,15 @@ class Chef
       end
       attr_accessor :cron_exists, :cron_empty
 
+      def whyrun_supported?
+        true
+      end
+
       def load_current_resource
         crontab_lines = []
         @current_resource = Chef::Resource::Cron.new(@new_resource.name)
         @current_resource.user(@new_resource.user)
+        @cron_exists = false
         if crontab = read_crontab
           cron_found = false
           crontab.each_line do |line|
@@ -89,14 +94,7 @@ class Chef
         newcron = String.new
         cron_found = false
 
-        newcron << "# Chef Name: #{new_resource.name}\n"
-        [ :mailto, :path, :shell, :home ].each do |v|
-          newcron << "#{v.to_s.upcase}=#{@new_resource.send(v)}\n" if @new_resource.send(v)
-        end
-        @new_resource.environment.each do |name, value|
-          newcron << "#{name}=#{value}\n"
-        end
-        newcron << "#{@new_resource.minute} #{@new_resource.hour} #{@new_resource.day} #{@new_resource.month} #{@new_resource.weekday} #{@new_resource.command}\n"
+        newcron = get_crontab_entry
 
         if @cron_exists
           unless cron_different?
@@ -125,20 +123,23 @@ class Chef
             end
             crontab << line
           end
+
           # Handle edge case where the Chef comment is the last line in the current crontab
           crontab << newcron if cron_found
 
-          write_crontab crontab
-          Chef::Log.info("#{@new_resource} updated crontab entry")
-          @new_resource.updated_by_last_action(true)
+          converge_by("update crontab entry for #{@new_resource}") do
+            write_crontab crontab
+            Chef::Log.info("#{@new_resource} updated crontab entry")
+          end
+
         else
           crontab = read_crontab unless @cron_empty
-
           crontab << newcron
 
-          write_crontab crontab
-          Chef::Log.info("#{@new_resource} added crontab entry")
-          @new_resource.updated_by_last_action(true)
+          converge_by("add crontab entry for #{@new_resource}") do
+            write_crontab crontab
+            Chef::Log.info("#{@new_resource} added crontab entry")
+          end
         end
       end
 
@@ -164,10 +165,12 @@ class Chef
             end
             crontab << line
           end
-
-          write_crontab crontab
-          Chef::Log.info("#{@new_resource} deleted crontab entry")
-          @new_resource.updated_by_last_action(true)
+          description = cron_found ? "remove #{@new_resource.name} from crontab" :
+            "save unmodified crontab"
+          converge_by(description) do
+            write_crontab crontab
+            Chef::Log.info("#{@new_resource} deleted crontab entry")
+          end
         end
       end
 
@@ -193,13 +196,33 @@ class Chef
       end
 
       def write_crontab(crontab)
+        write_exception = false
         status = popen4("crontab -u #{@new_resource.user} -", :waitlast => true) do |pid, stdin, stdout, stderr|
-          stdin.write crontab
+          begin
+            stdin.write crontab
+          rescue Errno::EPIPE => e
+            # popen4 could yield while child has already died.
+            write_exception = true
+            Chef::Log.debug("#{e.message}")
+          end
         end
-        if status.exitstatus > 0
+        if status.exitstatus > 0 || write_exception
           raise Chef::Exceptions::Cron, "Error updating state of #{@new_resource.name}, exit: #{status.exitstatus}"
         end
       end
+
+      def get_crontab_entry
+        newcron = ""
+        newcron << "# Chef Name: #{new_resource.name}\n"
+        [ :mailto, :path, :shell, :home ].each do |v|
+          newcron << "#{v.to_s.upcase}=#{@new_resource.send(v)}\n" if @new_resource.send(v)
+        end
+        @new_resource.environment.each do |name, value|
+          newcron << "#{name}=#{value}\n"
+        end
+        newcron << "#{@new_resource.minute} #{@new_resource.hour} #{@new_resource.day} #{@new_resource.month} #{@new_resource.weekday} #{@new_resource.command}\n"
+        newcron
+      end
     end
   end
 end
