@@ -6,9 +6,9 @@
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
-# 
+#
 #     http://www.apache.org/licenses/LICENSE-2.0
-# 
+#
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS,
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -22,18 +22,32 @@ class Chef
   class Provider
     class Group
       class Usermod < Chef::Provider::Group::Groupadd
-        
+
         def load_current_resource
           super
+        end
 
-          raise Chef::Exceptions::Group, "Could not find binary /usr/sbin/usermod for #{@new_resource}" unless ::File.exists?("/usr/sbin/usermod")
+        def define_resource_requirements
+          super
+
+          requirements.assert(:all_actions) do |a|
+            a.assertion { ::File.exists?("/usr/sbin/usermod") }
+            a.failure_message Chef::Exceptions::Group, "Could not find binary /usr/sbin/usermod for #{@new_resource}"
+            # No whyrun alternative: this component should be available in the base install of any given system that uses it
+          end
+
+          requirements.assert(:modify, :create) do |a|
+            a.assertion { @new_resource.members.empty? || @new_resource.append }
+            a.failure_message Chef::Exceptions::Group, "setting group members directly is not supported by #{self.to_s}, must set append true in group"
+            # No whyrun alternative - this action is simply not supported.
+          end
         end
 
         def modify_group_members
           case node[:platform]
-          when "openbsd", "netbsd", "aix"
+          when "openbsd", "netbsd", "aix", "solaris2", "smartos"
             append_flags = "-G"
-          when "solaris"
+          when "solaris", "suse", "opensuse"
             append_flags = "-a -G"
           end
 
@@ -42,10 +56,7 @@ class Chef
               @new_resource.members.each do |member|
                 Chef::Log.debug("#{@new_resource} appending member #{member} to group #{@new_resource.group_name}")
                 run_command(:command => "usermod #{append_flags} #{@new_resource.group_name} #{member}" )
-
               end
-            else
-              raise Chef::Exceptions::Group, "setting group members directly is not supported by #{self.to_s}"
             end
           else
             Chef::Log.debug("#{@new_resource} not changing group members, the group has no members")
