@@ -31,7 +31,14 @@ require 'rubygems/version'
 require 'rubygems/dependency'
 require 'rubygems/spec_fetcher'
 require 'rubygems/platform'
-require 'rubygems/format'
+
+# Compatibility note: Rubygems 2.0 removes rubygems/format in favor of
+# rubygems/package.
+begin
+  require 'rubygems/format'
+rescue LoadError
+  require 'rubygems/package'
+end
 require 'rubygems/dependency_installer'
 require 'rubygems/uninstaller'
 require 'rubygems/specification'
@@ -65,7 +72,7 @@ class Chef
             raise NotImplementedError
           end
 
-          ## 
+          ##
           # A rubygems specification object containing the list of gemspecs for all
           # available gems in the gem installation.
           # Implemented by subclasses
@@ -105,6 +112,22 @@ class Chef
             Gem.sources = original_sources
           end
 
+          ##
+          # Extracts the gemspec from a (on-disk) gem package.
+          # === Returns
+          # Gem::Specification
+          #
+          #--
+          # Compatibility note: Rubygems 1.x uses Gem::Format, 2.0 moved this
+          # code into Gem::Package.
+          def spec_from_file(file)
+            if defined?(Gem::Format) and Gem::Package.respond_to?(:open)
+              Gem::Format.from_file_by_path(file).spec
+            else
+              Gem::Package.new(file).spec
+            end
+          end
+
           ##
           # Determines the candidate version for a gem from a .gem file on disk
           # and checks if it matches the version contraints in +gem_dependency+
@@ -114,7 +137,7 @@ class Chef
           # nil           returns nil if the gem on disk doesn't match the
           #               version constraints for +gem_dependency+
           def candidate_version_from_file(gem_dependency, source)
-            spec = Gem::Format.from_file_by_path(source).spec
+            spec = spec_from_file(source)
             if spec.satisfies_requirement?(gem_dependency)
               logger.debug {"#{@new_resource} found candidate gem version #{spec.version} from local gem package #{source}"}
               spec.version
@@ -142,17 +165,26 @@ class Chef
           # Find the newest gem version available from Gem.sources that satisfies
           # the constraints of +gem_dependency+
           def find_newest_remote_version(gem_dependency, *sources)
-            # DependencyInstaller sorts the results such that the last one is
-            # always the one it considers best.
-            spec_with_source = dependency_installer.find_gems_with_sources(gem_dependency).last
+            available_gems = dependency_installer.find_gems_with_sources(gem_dependency)
+            spec, source = if available_gems.respond_to?(:last)
+              # DependencyInstaller sorts the results such that the last one is
+              # always the one it considers best.
+              spec_with_source = available_gems.last
+              spec_with_source && spec_with_source
+            else
+              # Rubygems 2.0 returns a Gem::Available set, which is a
+              # collection of AvailableSet::Tuple structs
+              available_gems.pick_best!
+              best_gem = available_gems.set.first
+              best_gem && [best_gem.spec, best_gem.source]
+            end
 
-            spec = spec_with_source && spec_with_source[0]
-            version = spec && spec_with_source[0].version
+            version = spec && spec.version
             if version
-              logger.debug { "#{@new_resource} found gem #{spec.name} version #{version} for platform #{spec.platform} from #{spec_with_source[1]}" }
+              logger.debug { "#{@new_resource} found gem #{spec.name} version #{version} for platform #{spec.platform} from #{source}" }
               version
             else
-              source_list = sources.compact.empty? ? "[#{Gem.sources.join(', ')}]" : "[#{sources.join(', ')}]"
+              source_list = sources.compact.empty? ? "[#{Gem.sources.to_a.join(', ')}]" : "[#{sources.join(', ')}]"
               logger.warn { "#{@new_resource} failed to find gem #{gem_dependency} from #{source_list}" }
               nil
             end
@@ -323,6 +355,7 @@ class Chef
         include Chef::Mixin::ShellOut
 
         attr_reader :gem_env
+        attr_reader :cleanup_gem_env
 
         def logger
           Chef::Log.logger
@@ -332,6 +365,7 @@ class Chef
 
         def initialize(new_resource, run_context=nil)
           super
+          @cleanup_gem_env = true
           if new_resource.gem_binary
             if new_resource.options && new_resource.options.kind_of?(Hash)
               msg =  "options cannot be given as a hash when using an explicit gem_binary\n"
@@ -343,12 +377,18 @@ class Chef
           elsif is_omnibus? && (!@new_resource.instance_of? Chef::Resource::ChefGem)
             # Opscode Omnibus - The ruby that ships inside omnibus is only used for Chef
             # Default to installing somewhere more functional
+            if new_resource.options && new_resource.options.kind_of?(Hash)
+              msg = "options should be a string instead of a hash\n"
+              msg << "in #{new_resource} from #{new_resource.source_line}"
+              raise ArgumentError, msg
+            end
             gem_location = find_gem_by_path
             @new_resource.gem_binary gem_location
             @gem_env = AlternateGemEnvironment.new(gem_location)
             Chef::Log.debug("#{@new_resource} using gem '#{gem_location}'")
           else
             @gem_env = CurrentGemEnvironment.new
+            @cleanup_gem_env = false
             Chef::Log.debug("#{@new_resource} using gem from running ruby environment")
           end
         end
@@ -385,6 +425,9 @@ class Chef
           # URI.parse gets confused by MS Windows paths with forward slashes.
           scheme = nil if scheme =~ /^[a-z]$/
           %w{http https}.include?(scheme)
+        rescue URI::InvalidURIError
+          Chef::Log.debug("#{@new_resource} failed to parse source '#{@new_resource.source}' as a URI, assuming a local path")
+          false
         end
 
         def current_version
@@ -430,6 +473,13 @@ class Chef
           @current_resource
         end
 
+        def cleanup_after_converge
+          if @cleanup_gem_env
+            logger.debug { "#{@new_resource} resetting gem environment to default" }
+            Gem.clear_paths
+          end
+        end
+
         def candidate_version
           @candidate_version ||= begin
             if target_version_already_installed?
@@ -487,7 +537,7 @@ class Chef
           if version
             shell_out!("#{gem_binary_path} install #{name} -q --no-rdoc --no-ri -v \"#{version}\"#{src}#{opts}", :env=>nil)
           else
-            shell_out!("#{gem_binary_path} install #{name} -q --no-rdoc --no-ri #{src}#{opts}", :env=>nil)
+            shell_out!("#{gem_binary_path} install \"#{name}\" -q --no-rdoc --no-ri #{src}#{opts}", :env=>nil)
           end
         end
 
