@@ -16,8 +16,10 @@
 # limitations under the License.
 #
 
+require 'chef/config'
 require 'chef/provider/package'
 require 'chef/mixin/command'
+require 'chef/mixin/shell_out'
 require 'chef/resource/package'
 require 'singleton'
 require 'chef/mixin/get_source_from_package'
@@ -645,6 +647,7 @@ class Chef
         # Cache for our installed and available packages, pulled in from yum-dump.py
         class YumCache
           include Chef::Mixin::Command
+          include Chef::Mixin::ShellOut
           include Singleton
 
           def initialize
@@ -664,12 +667,16 @@ class Chef
 
             @allow_multi_install = []
 
+            @extra_repo_control = nil
+
             # these are for subsequent runs if we are on an interval
             Chef::Client.when_run_starts do
               YumCache.instance.reload
             end
           end
 
+          attr_reader :extra_repo_control
+
           # Cache management
           #
 
@@ -693,13 +700,19 @@ class Chef
               raise ArgumentError, "Unexpected value in next_refresh: #{@next_refresh}"
             end
 
+            if @extra_repo_control
+              opts << " #{@extra_repo_control}"
+            end
+
             one_line = false
             error = nil
 
             helper = ::File.join(::File.dirname(__FILE__), 'yum-dump.py')
+            status = nil
 
-            status = popen4("/usr/bin/python #{helper}#{opts}", :waitlast => true) do |pid, stdin, stdout, stderr|
-              stdout.each do |line|
+            begin
+              status = shell_out!("/usr/bin/python #{helper}#{opts}", :timeout => Chef::Config[:yum_timeout])
+              status.stdout.each_line do |line|
                 one_line = true
 
                 line.chomp!
@@ -746,7 +759,10 @@ class Chef
                 @rpmdb << pkg
               end
 
-              error = stderr.readlines
+              error = status.stderr
+            rescue Mixlib::ShellOut::CommandTimeout => e
+              Chef::Log.error("#{helper} exceeded timeout #{Chef::Config[:yum_timeout]}")
+              raise(e)
             end
 
             if status.exitstatus != 0
@@ -848,6 +864,22 @@ class Chef
             @allow_multi_install
           end
 
+          def enable_extra_repo_control(arg)
+            # Don't touch cache if it's the same repos as the last load
+            unless @extra_repo_control == arg
+              @extra_repo_control = arg
+              reload
+            end
+          end
+
+          def disable_extra_repo_control
+            # Only force reload when set
+            if @extra_repo_control
+              @extra_repo_control = nil
+              reload
+            end
+          end
+
           private
 
           def version(package_name, arch=nil, is_available=false, is_installed=false)
@@ -915,6 +947,7 @@ class Chef
         end # YumCache
 
         include Chef::Mixin::GetSourceFromPackage
+        include Chef::Mixin::ShellOut
 
         def initialize(new_resource, run_context)
           super
@@ -957,7 +990,7 @@ class Chef
         end
 
         def yum_command(command)
-          status, stdout, stderr = output_of_command(command, {})
+          status, stdout, stderr = output_of_command(command, {:timeout => Chef::Config[:yum_timeout]})
 
           # This is fun: rpm can encounter errors in the %post/%postun scripts which aren't
           # considered fatal - meaning the rpm is still successfully installed. These issue
@@ -974,7 +1007,7 @@ class Chef
               if l =~ %r{^error: %(post|postun)\(.*\) scriptlet failed, exit status \d+$}
                 Chef::Log.warn("#{@new_resource} caught non-fatal scriptlet issue: \"#{l}\". Can't trust yum exit status " +
                                "so running install again to verify.")
-                status, stdout, stderr = output_of_command(command, {})
+                status, stdout, stderr = output_of_command(command, {:timeout => Chef::Config[:yum_timeout]})
                 break
               end
             end
@@ -995,8 +1028,25 @@ class Chef
             @yum.reload
           end
 
+          if @new_resource.options
+            repo_control = []
+            @new_resource.options.split.each do |opt|
+              if opt =~ %r{--(enable|disable)repo=.+}
+                repo_control << opt
+              end
+            end
+
+            if repo_control.size > 0
+              @yum.enable_extra_repo_control(repo_control.join(" "))
+            else
+              @yum.disable_extra_repo_control
+            end
+          else
+            @yum.disable_extra_repo_control
+          end
+
           # At this point package_name could be:
-          # 
+          #
           # 1) a package name, eg: "foo"
           # 2) a package name.arch, eg: "foo.i386"
           # 3) or a dependency, eg: "foo >= 1.1"
@@ -1021,13 +1071,11 @@ class Chef
             end
 
             Chef::Log.debug("#{@new_resource} checking rpm status")
-            status = popen4("rpm -qp --queryformat '%{NAME} %{VERSION}-%{RELEASE}\n' #{@new_resource.source}") do |pid, stdin, stdout, stderr|
-              stdout.each do |line|
-                case line
-                when /([\w\d_.-]+)\s([\w\d_.-]+)/
-                  @current_resource.package_name($1)
-                  @new_resource.version($2)
-                end
+            shell_out!("rpm -qp --queryformat '%{NAME} %{VERSION}-%{RELEASE}\n' #{@new_resource.source}", :timeout => Chef::Config[:yum_timeout]).stdout.each_line do |line|
+              case line
+              when /([\w\d_.-]+)\s([\w\d_.-]+)/
+                @current_resource.package_name($1)
+                @new_resource.version($2)
               end
             end
           end
@@ -1106,7 +1154,7 @@ class Chef
         # Hacky - better overall solution? Custom compare in Package provider?
         def action_upgrade
           # Could be uninstalled or have no candidate
-          if @current_resource.version.nil? || candidate_version.nil? 
+          if @current_resource.version.nil? || candidate_version.nil?
             super
           # Ensure the candidate is newer
           elsif RPMVersion.parse(candidate_version) > RPMVersion.parse(@current_resource.version)
