@@ -16,10 +16,12 @@
 # limitations under the License.
 #
 
+require 'chef/provider/batch'
 require 'chef/provider/breakpoint'
 require 'chef/provider/cookbook_file'
 require 'chef/provider/cron'
 require 'chef/provider/cron/solaris'
+require 'chef/provider/cron/aix'
 require 'chef/provider/deploy'
 require 'chef/provider/directory'
 require 'chef/provider/env'
@@ -36,6 +38,7 @@ require 'chef/provider/ohai'
 require 'chef/provider/mdadm'
 require 'chef/provider/mount'
 require 'chef/provider/package'
+require 'chef/provider/powershell_script'
 require 'chef/provider/remote_directory'
 require 'chef/provider/remote_file'
 require 'chef/provider/route'
@@ -52,6 +55,7 @@ require 'chef/provider/package/apt'
 require 'chef/provider/package/dpkg'
 require 'chef/provider/package/easy_install'
 require 'chef/provider/package/freebsd'
+require 'chef/provider/package/ips'
 require 'chef/provider/package/macports'
 require 'chef/provider/package/pacman'
 require 'chef/provider/package/portage'
@@ -61,6 +65,7 @@ require 'chef/provider/package/yum'
 require 'chef/provider/package/zypper'
 require 'chef/provider/package/solaris'
 require 'chef/provider/package/smartos'
+require 'chef/provider/package/aix'
 
 require 'chef/provider/service/arch'
 require 'chef/provider/service/debian'
@@ -81,18 +86,38 @@ require 'chef/provider/user/dscl'
 require 'chef/provider/user/pw'
 require 'chef/provider/user/useradd'
 require 'chef/provider/user/windows'
+require 'chef/provider/user/solaris'
 
 require 'chef/provider/group/aix'
 require 'chef/provider/group/dscl'
 require 'chef/provider/group/gpasswd'
 require 'chef/provider/group/groupadd'
+require 'chef/provider/group/groupmod'
 require 'chef/provider/group/pw'
 require 'chef/provider/group/suse'
 require 'chef/provider/group/usermod'
 require 'chef/provider/group/windows'
 
 require 'chef/provider/mount/mount'
+require 'chef/provider/mount/aix'
 require 'chef/provider/mount/windows'
 
 require 'chef/provider/deploy/revision'
 require 'chef/provider/deploy/timestamped'
+
+require 'chef/provider/remote_file/ftp'
+require 'chef/provider/remote_file/http'
+require 'chef/provider/remote_file/local_file'
+require 'chef/provider/remote_file/fetcher'
+
+require "chef/provider/lwrp_base"
+require 'chef/provider/registry_key'
+
+require 'chef/provider/file/content'
+require 'chef/provider/remote_file/content'
+require 'chef/provider/cookbook_file/content'
+require 'chef/provider/template/content'
+
+require 'chef/provider/ifconfig/redhat'
+require 'chef/provider/ifconfig/debian'
+require 'chef/provider/ifconfig/aix'
