@@ -18,10 +18,12 @@
 #
 
 require 'chef/mixin/params_validate'
-require 'chef/mixin/check_helper'
-require 'chef/mixin/language'
+require 'chef/dsl/platform_introspection'
+require 'chef/dsl/data_query'
+require 'chef/dsl/registry_helper'
 require 'chef/mixin/convert_to_class_name'
 require 'chef/resource/conditional'
+require 'chef/resource/conditional_action_not_nothing'
 require 'chef/resource_collection'
 require 'chef/resource_platform_map'
 require 'chef/node'
@@ -41,19 +43,34 @@ class Chef
         other_notification.resource == resource && other_notification.action == action
       end
 
+      # If resource and/or notifying_resource is not a resource object, this will look them up in the resource collection
+      # and fix the references from strings to actual Resource objects.
       def resolve_resource_reference(resource_collection)
-        return resource if resource.kind_of?(Chef::Resource)
+        return resource if resource.kind_of?(Chef::Resource) && notifying_resource.kind_of?(Chef::Resource)
 
+        if not(resource.kind_of?(Chef::Resource))
+          fix_resource_reference(resource_collection)
+        end
+
+        if not(notifying_resource.kind_of?(Chef::Resource))
+          fix_notifier_reference(resource_collection)
+        end
+      end
+
+      # This will look up the resource if it is not a Resource Object.  It will complain if it finds multiple
+      # resources, can't find a resource, or gets invalid syntax.
+      def fix_resource_reference(resource_collection)
         matching_resource = resource_collection.find(resource)
         if Array(matching_resource).size > 1
           msg = "Notification #{self} from #{notifying_resource} was created with a reference to multiple resources, "\
-                "but can only notify one resource. Notifying resource was defined on #{notifying_resource.source_line}"
+          "but can only notify one resource. Notifying resource was defined on #{notifying_resource.source_line}"
           raise Chef::Exceptions::InvalidResourceReference, msg
         end
         self.resource = matching_resource
+
       rescue Chef::Exceptions::ResourceNotFound => e
         err = Chef::Exceptions::ResourceNotFound.new(<<-FAIL)
-Resource #{notifying_resource} is configured to notify resource #{resource} with action #{action}, \
+resource #{notifying_resource} is configured to notify resource #{resource} with action #{action}, \
 but #{resource} cannot be found in the resource collection. #{notifying_resource} is defined in \
 #{notifying_resource.source_line}
 FAIL
@@ -64,6 +81,36 @@ FAIL
 Resource #{notifying_resource} is configured to notify resource #{resource} with action #{action}, \
 but #{resource.inspect} is not valid syntax to look up a resource in the resource collection. Notification \
 is defined near #{notifying_resource.source_line}
+F
+          err.set_backtrace(e.backtrace)
+        raise err
+      end
+
+      # This will look up the notifying_resource if it is not a Resource Object.  It will complain if it finds multiple
+      # resources, can't find a resource, or gets invalid syntax.
+      def fix_notifier_reference(resource_collection)
+        matching_notifier = resource_collection.find(notifying_resource)
+        if Array(matching_notifier).size > 1
+          msg = "Notification #{self} from #{notifying_resource} was created with a reference to multiple notifying "\
+          "resources, but can only originate from one resource.  Destination resource was defined "\
+          "on #{resource.source_line}"
+          raise Chef::Exceptions::InvalidResourceReference, msg
+        end
+        self.notifying_resource = matching_notifier
+
+      rescue Chef::Exceptions::ResourceNotFound => e
+        err = Chef::Exceptions::ResourceNotFound.new(<<-FAIL)
+Resource #{resource} is configured to receive notifications from #{notifying_resource} with action #{action}, \
+but #{notifying_resource} cannot be found in the resource collection. #{resource} is defined in \
+#{resource.source_line}
+FAIL
+        err.set_backtrace(e.backtrace)
+        raise err
+      rescue Chef::Exceptions::InvalidResourceSpecification => e
+          err = Chef::Exceptions::InvalidResourceSpecification.new(<<-F)
+Resource #{resource} is configured to receive notifications from  #{notifying_resource} with action #{action}, \
+but #{notifying_resource.inspect} is not valid syntax to look up a resource in the resource collection. Notification \
+is defined near #{resource.source_line}
 F
           err.set_backtrace(e.backtrace)
         raise err
@@ -71,15 +118,95 @@ F
 
     end
 
-    FORBIDDEN_IVARS = [:@run_context, :@node, :@not_if, :@only_if]
-    HIDDEN_IVARS = [:@allowed_actions, :@resource_name, :@source_line, :@run_context, :@name, :@node]
+    FORBIDDEN_IVARS = [:@run_context, :@node, :@not_if, :@only_if, :@enclosing_provider]
+    HIDDEN_IVARS = [:@allowed_actions, :@resource_name, :@source_line, :@run_context, :@name, :@node, :@not_if, :@only_if, :@elapsed_time, :@enclosing_provider]
 
-    include Chef::Mixin::CheckHelper
+    include Chef::DSL::DataQuery
     include Chef::Mixin::ParamsValidate
-    include Chef::Mixin::Language
+    include Chef::DSL::PlatformIntrospection
+    include Chef::DSL::RegistryHelper
     include Chef::Mixin::ConvertToClassName
     include Chef::Mixin::Deprecation
 
+    extend Chef::Mixin::ConvertToClassName
+
+
+    if Module.method(:const_defined?).arity == 1
+      def self.strict_const_defined?(const)
+        const_defined?(const)
+      end
+    else
+      def self.strict_const_defined?(const)
+        const_defined?(const, false)
+      end
+    end
+
+    # Track all subclasses of Resource. This is used so names can be looked up
+    # when attempting to deserialize from JSON. (See: json_compat)
+    def self.resource_classes
+      # Using a class variable here ensures we have one variable to track
+      # subclasses shared by the entire class hierarchy; without this, each
+      # subclass would have its own list of subclasses.
+      @@resource_classes ||= []
+    end
+
+    # Callback when subclass is defined. Adds subclass to list of subclasses.
+    def self.inherited(subclass)
+      resource_classes << subclass
+    end
+
+    # Look up a subclass by +class_name+ which should be a string that matches
+    # `Subclass.name`
+    def self.find_subclass_by_name(class_name)
+      resource_classes.first {|c| c.name == class_name }
+    end
+
+    # Set or return the list of "state attributes" implemented by the Resource
+    # subclass. State attributes are attributes that describe the desired state
+    # of the system, such as file permissions or ownership. In general, state
+    # attributes are attributes that could be populated by examining the state
+    # of the system (e.g., File.stat can tell you the permissions on an
+    # existing file). Contrarily, attributes that are not "state attributes"
+    # usually modify the way Chef itself behaves, for example by providing
+    # additional options for a package manager to use when installing a
+    # package.
+    #
+    # This list is used by the Chef client auditing system to extract
+    # information from resources to describe changes made to the system.
+    def self.state_attrs(*attr_names)
+      @state_attrs ||= []
+      @state_attrs = attr_names unless attr_names.empty?
+
+      # Return *all* state_attrs that this class has, including inherited ones
+      if superclass.respond_to?(:state_attrs)
+        superclass.state_attrs + @state_attrs
+      else
+        @state_attrs
+      end
+    end
+
+    # Set or return the "identity attribute" for this resource class. This is
+    # generally going to be the "name attribute" for this resource. In other
+    # words, the resource type plus this attribute uniquely identify a given
+    # bit of state that chef manages. For a File resource, this would be the
+    # path, for a package resource, it will be the package name. This will show
+    # up in chef-client's audit records as a searchable field.
+    def self.identity_attr(attr_name=nil)
+      @identity_attr ||= nil
+      @identity_attr = attr_name if attr_name
+
+      # If this class doesn't have an identity attr, we'll defer to the superclass:
+      if @identity_attr || !superclass.respond_to?(:identity_attr)
+        @identity_attr
+      else
+        superclass.identity_attr
+      end
+    end
+
+    def self.dsl_name
+      convert_to_snake_case(name, 'Chef::Resource')
+    end
+
     attr_accessor :params
     attr_accessor :provider
     attr_accessor :allowed_actions
@@ -97,10 +224,10 @@ F
     attr_reader :not_if_args
     attr_reader :only_if_args
 
+    attr_reader :elapsed_time
+
     # Each notify entry is a resource/action pair, modeled as an
     # Struct with a #resource and #action member
-    attr_reader :immediate_notifications
-    attr_reader :delayed_notifications
 
     def initialize(name, run_context=nil)
       @name = name
@@ -119,13 +246,32 @@ F
       @retry_delay = 2
       @not_if = []
       @only_if = []
-      @immediate_notifications = Array.new
-      @delayed_notifications = Array.new
       @source_line = nil
+      @elapsed_time = 0
 
       @node = run_context ? deprecated_ivar(run_context.node, :node, :warn) : nil
     end
 
+    # Returns a Hash of attribute => value for the state attributes declared in
+    # the resource's class definition.
+    def state
+      self.class.state_attrs.inject({}) do |state_attrs, attr_name|
+        state_attrs[attr_name] = send(attr_name)
+        state_attrs
+      end
+    end
+
+    # Returns the value of the identity attribute, if declared. Falls back to
+    # #name if no identity attribute is declared.
+    def identity
+      if identity_attr = self.class.identity_attr
+        send(identity_attr)
+      else
+        name
+      end
+    end
+
+
     def updated=(true_or_false)
       Chef::Log.warn("Chef::Resource#updated=(true|false) is deprecated. Please call #updated_by_last_action(true|false) instead.")
       Chef::Log.warn("Called from:")
@@ -152,14 +298,18 @@ F
     def load_prior_resource
       begin
         prior_resource = run_context.resource_collection.lookup(self.to_s)
-        Chef::Log.debug("Setting #{self.to_s} to the state of the prior #{self.to_s}")
+        # if we get here, there is a prior resource (otherwise we'd have jumped
+        # to the rescue clause).
+        Chef::Log.warn("Cloning resource attributes for #{self.to_s} from prior resource (CHEF-3694)")
+        Chef::Log.warn("Previous #{prior_resource}: #{prior_resource.source_line}") if prior_resource.source_line
+        Chef::Log.warn("Current  #{self}: #{self.source_line}") if self.source_line
         prior_resource.instance_variables.each do |iv|
-          unless iv.to_sym == :@source_line || iv.to_sym == :@action
+          unless iv.to_sym == :@source_line || iv.to_sym == :@action || iv.to_sym == :@not_if || iv.to_sym == :@only_if
             self.instance_variable_set(iv, prior_resource.instance_variable_get(iv))
           end
         end
         true
-      rescue Chef::Exceptions::ResourceNotFound => e
+      rescue Chef::Exceptions::ResourceNotFound
         true
       end
     end
@@ -206,17 +356,19 @@ F
     end
 
     def name(name=nil)
-      set_if_args(@name, name) do
+      if !name.nil?
         raise ArgumentError, "name must be a string!" unless name.kind_of?(String)
         @name = name
       end
+      @name
     end
 
     def noop(tf=nil)
-      set_if_args(@noop, tf) do
+      if !tf.nil?
         raise ArgumentError, "noop must be true or false!" unless tf == true || tf == false
         @noop = tf
       end
+      @noop
     end
 
     def ignore_failure(arg=nil)
@@ -247,32 +399,15 @@ F
       ignore_failure(arg)
     end
 
-    def notifies(*args)
-      unless ( args.size > 0 && args.size < 4)
-        raise ArgumentError, "Wrong number of arguments for notifies: should be 1-3 arguments, you gave #{args.inspect}"
-      end
-
-      if args.size > 1 # notifies(:action, resource) OR notifies(:action, resource, :immediately)
-        add_notification(*args)
-      else
-        # This syntax is so weird. surely people will just give us one hash?
-        notifications = args.flatten
-        notifications.each do |resources_notifications|
-          resources_notifications.each do |resource, notification|
-            action, timing = notification[0], notification[1]
-            Chef::Log.debug "Adding notification from resource #{self} to `#{resource.inspect}' => `#{notification.inspect}'"
-            add_notification(action, resource, timing)
-          end
-        end
-      end
-    rescue NoMethodError
-      Chef::Log.fatal("Error processing notifies(#{args.inspect}) on #{self}")
-      raise
-    end
-
-    def add_notification(action, resources, timing=:delayed)
-      resources = [resources].flatten
+    # Sets up a notification from this resource to the resource specified by +resource_spec+.
+    def notifies(action, resource_spec, timing=:delayed)
+      # when using old-style resources(:template => "/foo.txt") style, you
+      # could end up with multiple resources.
+      resources = [ resource_spec ].flatten
       resources.each do |resource|
+
+        validate_resource_spec!(resource_spec)
+
         case timing.to_s
         when 'delayed'
           notifies_delayed(action, resource)
@@ -291,16 +426,24 @@ F
     # resolve_resource_reference on each in turn, causing them to
     # resolve lazy/forward references.
     def resolve_notification_references
-      @immediate_notifications.each { |n| n.resolve_resource_reference(run_context.resource_collection) }
-      @delayed_notifications.each {|n| n.resolve_resource_reference(run_context.resource_collection) }
+      run_context.immediate_notifications(self).each { |n| n.resolve_resource_reference(run_context.resource_collection) }
+      run_context.delayed_notifications(self).each {|n| n.resolve_resource_reference(run_context.resource_collection) }
     end
 
     def notifies_immediately(action, resource_spec)
-      @immediate_notifications << Notification.new(resource_spec, action, self)
+      run_context.notifies_immediately(Notification.new(resource_spec, action, self))
     end
 
     def notifies_delayed(action, resource_spec)
-      @delayed_notifications << Notification.new(resource_spec, action, self)
+      run_context.notifies_delayed(Notification.new(resource_spec, action, self))
+    end
+
+    def immediate_notifications
+      run_context.immediate_notifications(self)
+    end
+
+    def delayed_notifications
+      run_context.delayed_notifications(self)
     end
 
     def resources(*args)
@@ -310,11 +453,21 @@ F
     def subscribes(action, resources, timing=:delayed)
       resources = [resources].flatten
       resources.each do |resource|
+        if resource.is_a?(String)
+          resource = Chef::Resource.new(resource, run_context)
+        end
+        if resource.run_context.nil?
+          resource.run_context = run_context
+        end
         resource.notifies(action, self, timing)
       end
       true
     end
 
+    def validate_resource_spec!(resource_spec)
+      run_context.resource_collection.validate_lookup_spec!(resource_spec)
+    end
+
     def is(*args)
       if args.size == 1
         args.first
@@ -329,13 +482,17 @@ F
 
     def to_text
       ivars = instance_variables.map { |ivar| ivar.to_sym } - HIDDEN_IVARS
-      text = "# Declared in #{@source_line}\n"
-      text << convert_to_snake_case(self.class.name, 'Chef::Resource') + "(\"#{name}\") do\n"
+      text = "# Declared in #{@source_line}\n\n"
+      text << self.class.dsl_name + "(\"#{name}\") do\n"
       ivars.each do |ivar|
         if (value = instance_variable_get(ivar)) && !(value.respond_to?(:empty?) && value.empty?)
-          text << "  #{ivar.to_s.sub(/^@/,'')}(#{value.inspect})\n"
+          value_string = value.respond_to?(:to_text) ? value.to_text : value.inspect
+          text << "  #{ivar.to_s.sub(/^@/,'')} #{value_string}\n"
         end
       end
+      [@not_if, @only_if].flatten.each do |conditional|
+        text << "  #{conditional.to_text}\n"
+      end
       text << "end\n"
     end
 
@@ -355,7 +512,7 @@ F
       safe_ivars.each do |iv|
         instance_vars[iv.to_s.sub(/^@/, '')] = instance_variable_get(iv)
       end
-      results = {
+      {
         'json_class' => self.class.name,
         'instance_vars' => instance_vars
       }
@@ -420,17 +577,40 @@ F
     end
 
     def defined_at
+      # The following regexp should match these two sourceline formats:
+      #   /some/path/to/file.rb:80:in `wombat_tears'
+      #   C:/some/path/to/file.rb:80 in 1`wombat_tears'
+      # extracting the path to the source file and the line number.
+      (file, line_no) = source_line.match(/(.*):(\d+):?.*$/).to_a[1,2] if source_line
       if cookbook_name && recipe_name && source_line
-        "#{cookbook_name}::#{recipe_name} line #{source_line.split(':')[1]}"
+        "#{cookbook_name}::#{recipe_name} line #{line_no}"
       elsif source_line
-        file, line_no = source_line.split(':')
         "#{file} line #{line_no}"
       else
         "dynamically defined"
       end
     end
 
-    def run_action(action)
+    def cookbook_version
+      if cookbook_name
+        run_context.cookbook_collection[cookbook_name]
+      end
+    end
+
+    def events
+      run_context.events
+    end
+
+    def run_action(action, notification_type=nil, notifying_resource=nil)
+      # reset state in case of multiple actions on the same resource.
+      @elapsed_time = 0
+      start_time = Time.now
+      events.resource_action_start(self, action, notification_type, notifying_resource)
+      # Try to resolve lazy/forward references in notifications again to handle
+      # the case where the resource was defined lazily (ie. in a ruby_block)
+      resolve_notification_references
+      validate_action(action)
+
       if Chef::Config[:verbose_logging] || Chef::Log.level == :debug
         # This can be noisy
         Chef::Log.info("Processing #{self} action #{action} (#{defined_at})")
@@ -441,29 +621,50 @@ F
       updated_by_last_action(false)
 
       begin
-        return if should_skip?
-        # leverage new platform => short_name => resource
-        # which requires explicitly setting provider in
-        # resource class
-        if self.provider
-          provider = self.provider.new(self, self.run_context)
-        else # fall back to old provider resolution
-          provider = Chef::Platform.provider_for_resource(self)
-        end
-        provider.load_current_resource
-        provider.send("action_#{action}")
-      rescue => e
+        return if should_skip?(action)
+        provider_for_action(action).run_action
+      rescue Exception => e
         if ignore_failure
-          Chef::Log.error("#{self} (#{defined_at}) had an error: #{e.message}")
+          Chef::Log.error("#{self} (#{defined_at}) had an error: #{e.message}; ignore_failure is set, continuing")
+          events.resource_failed(self, action, e)
+        elsif retries > 0
+          events.resource_failed_retriable(self, action, retries, e)
+          @retries -= 1
+          Chef::Log.info("Retrying execution of #{self}, #{retries} attempt(s) left")
+          sleep retry_delay
+          retry
         else
-          Chef::Log.error("#{self} (#{defined_at}) has had an error")
-          new_exception = e.exception("#{self} (#{defined_at}) had an error: #{e.class.name}: #{e.message}")
-          new_exception.set_backtrace(e.backtrace)
-          raise new_exception
+          events.resource_failed(self, action, e)
+          raise customize_exception(e)
         end
+      ensure
+        @elapsed_time = Time.now - start_time
+        events.resource_completed(self)
       end
     end
 
+    def validate_action(action)
+      raise ArgumentError, "nil is not a valid action for resource #{self}" if action.nil?
+    end
+
+    def provider_for_action(action)
+      # leverage new platform => short_name => resource
+      # which requires explicitly setting provider in
+      # resource class
+      if self.provider
+        provider = self.provider.new(self, self.run_context)
+        provider.action = action
+        provider
+      else # fall back to old provider resolution
+        Chef::Platform.provider_for_resource(self, action)
+      end
+    end
+
+    def customize_exception(e)
+      new_exception = e.exception("#{self} (#{defined_at}) had an error: #{e.class.name}: #{e.message}")
+      new_exception.set_backtrace(e.backtrace)
+      new_exception
+    end
     # Evaluates not_if and only_if conditionals. Returns a falsey value if any
     # of the conditionals indicate that this resource should be skipped, i.e.,
     # if an only_if evaluates to false or a not_if evaluates to true.
@@ -472,14 +673,17 @@ F
     # "fails" its check. Subsequent conditionals are not evaluated, so in
     # general it's not a good idea to rely on side effects from not_if or
     # only_if commands/blocks being evaluated.
-    def should_skip?
-      conditionals = only_if + not_if
-      return false if conditionals.empty?
+    #
+    # Also skips conditional checking when the action is :nothing
+    def should_skip?(action)
+      conditional_action = ConditionalActionNotNothing.new(action)
 
+      conditionals = [ conditional_action ] + only_if + not_if
       conditionals.find do |conditional|
         if conditional.continue?
           false
         else
+          events.resource_skipped(self, action, conditional)
           Chef::Log.debug("Skipping #{self} due to #{conditional.description}")
           true
         end
@@ -512,95 +716,6 @@ F
       nil
     end
 
-    extend Chef::Mixin::ConvertToClassName
-
-    def self.attribute(attr_name, validation_opts={})
-      # This atrocity is the only way to support 1.8 and 1.9 at the same time
-      # When you're ready to drop 1.8 support, do this:
-      # define_method attr_name.to_sym do |arg=nil|
-      # etc.
-      shim_method=<<-SHIM
-      def #{attr_name}(arg=nil)
-        _set_or_return_#{attr_name}(arg)
-      end
-      SHIM
-      class_eval(shim_method)
-
-      define_method("_set_or_return_#{attr_name.to_s}".to_sym) do |arg|
-        set_or_return(attr_name.to_sym, arg, validation_opts)
-      end
-    end
-
-    def self.build_from_file(cookbook_name, filename, run_context)
-      rname = filename_to_qualified_string(cookbook_name, filename)
-
-      # Add log entry if we override an existing light-weight resource.
-      class_name = convert_to_class_name(rname)
-      overriding = Chef::Resource.const_defined?(class_name)
-      Chef::Log.info("#{class_name} light-weight resource already initialized -- overriding!") if overriding
-
-      new_resource_class = Class.new self do |cls|
-
-        # default initialize method that ensures that when initialize is finally
-        # wrapped (see below), super is called in the event that the resource
-        # definer does not implement initialize
-        def initialize(name, run_context)
-          super(name, run_context)
-        end
-
-        @actions_to_create = []
-
-        class << cls
-          include Chef::Mixin::FromFile
-
-          attr_accessor :run_context
-          attr_reader :action_to_set_default
-
-          def node
-            self.run_context.node
-          end
-
-          def actions_to_create
-            @actions_to_create
-          end
-
-          define_method(:default_action) do |action_name|
-            actions_to_create.push(action_name)
-            @action_to_set_default = action_name
-          end
-
-          define_method(:actions) do |*action_names|
-            actions_to_create.push(*action_names)
-          end
-        end
-
-        # set the run context in the class instance variable
-        cls.run_context = run_context
-
-        # load resource definition from file
-        cls.class_from_file(filename)
-
-        # create a new constructor that wraps the old one and adds the actions
-        # specified in the DSL
-        old_init = instance_method(:initialize)
-
-        define_method(:initialize) do |name, *optional_args|
-          args_run_context = optional_args.shift
-          @resource_name = rname.to_sym
-          old_init.bind(self).call(name, args_run_context)
-          @action = self.class.action_to_set_default || @action
-          allowed_actions.push(self.class.actions_to_create).flatten!
-        end
-      end
-
-      # register new class as a Chef::Resource
-      class_name = convert_to_class_name(rname)
-      Chef::Resource.const_set(class_name, new_resource_class)
-      Chef::Log.debug("Loaded contents of #{filename} into a resource named #{rname} defined in Chef::Resource::#{class_name}")
-
-      new_resource_class
-    end
-
     # Resources that want providers namespaced somewhere other than
     # Chef::Provider can set the namespace with +provider_base+
     # Ex:
