@@ -1,5 +1,6 @@
 #
 # Author:: Daniel DeLeo (<dan@kallistec.com>)
+# Author:: Tyler Cloke (<tyler@opscode.com>)
 # Copyright:: Copyright (c) 2008 Opscode, Inc.
 # License:: Apache License, Version 2.0
 #
@@ -51,7 +52,11 @@ class Chef
     class Deploy < Chef::Resource
 
       provider_base Chef::Provider::Deploy
-      
+
+      identity_attr :repository
+
+      state_attrs :deploy_to, :revision
+
       def initialize(name, run_context=nil)
         super
         @resource_name = :deploy
@@ -75,6 +80,7 @@ class Chef
         @provider = Chef::Provider::Deploy::Timestamped
         @allowed_actions.push(:force_deploy, :deploy, :rollback)
         @additional_remotes = Hash[]
+        @keep_releases = 5
       end
 
       # where the checked out/cloned code goes
@@ -295,6 +301,14 @@ class Chef
         )
       end
 
+       # The number of old release directories to keep around after cleanup
+      def keep_releases(arg=nil)
+        [set_or_return(
+          :keep_releases,
+          arg,
+          :kind_of => [ Integer ]), 1].max
+      end
+
       # An array of paths, relative to your app's root, to be purged from a
       # SCM clone/checkout before symlinking. Use this to get rid of files and
       # directories you want to be shared between releases.
@@ -375,7 +389,7 @@ class Chef
         arg ||= block
         set_or_return(:after_restart, arg, :kind_of => [Proc, String])
       end
-      
+
       def additional_remotes(arg=nil)
         set_or_return(
           :additional_remotes,
@@ -384,6 +398,19 @@ class Chef
         )
       end
 
+      # FIXME The Deploy resource may be passed to an SCM provider as its
+      # resource.  The SCM provider knows that SCM resources can specify a
+      # timeout for SCM operations. The deploy resource must therefore support
+      # a timeout method, but the timeout it describes is for SCM operations,
+      # not the overall deployment. This is potentially confusing.
+      def timeout(arg=nil)
+        set_or_return(
+          :timeout,
+          arg,
+          :kind_of => Integer
+        )
+      end
+
     end
   end
 end
