@@ -1,5 +1,6 @@
 #
 # Author:: Adam Jacob (<adam@opscode.com>)
+# Author:: Tyler Cloke (<tyler@opscode.com>)
 # Copyright:: Copyright (c) 2008 Opscode, Inc.
 # License:: Apache License, Version 2.0
 #
@@ -24,9 +25,14 @@ class Chef
     class Link < Chef::Resource
       include Chef::Mixin::Securable
 
-       provides :link, :on_platform  => :all
+      provides :link, :on_platform  => :all
+
+      identity_attr :target_file
+
+      state_attrs :to, :owner, :group
 
       def initialize(name, run_context=nil)
+        verify_links_supported!
         super
         @resource_name = :link
         @to = nil
@@ -62,6 +68,42 @@ class Chef
         )
       end
 
+      def group(arg=nil)
+        set_or_return(
+          :group,
+          arg,
+          :regex => Chef::Config[:group_valid_regex]
+        )
+      end
+
+      def owner(arg=nil)
+        set_or_return(
+          :owner,
+          arg,
+          :regex => Chef::Config[:user_valid_regex]
+        )
+      end
+
+      # make link quack like a file (XXX: not for public consumption)
+      def path
+        @target_file
+      end
+
+      private
+      def verify_links_supported!
+        # On certain versions of windows links are not supported. Make
+        # sure we are not on such a platform.
+
+        if Chef::Platform.windows?
+          require 'chef/win32/file'
+          begin
+            Chef::ReservedNames::Win32::File.verify_links_supported!
+          rescue Chef::Exceptions::Win32APIFunctionNotImplemented => e
+            Chef::Log.fatal("Link resource is not supported on this version of Windows")
+            raise e
+          end
+        end
+      end
     end
   end
 end
