@@ -1,5 +1,6 @@
 #
 # Author:: Adam Jacob (<adam@opscode.com>)
+# Author:: Tyler Cloke (<tyler@opscode.com>)
 # Copyright:: Copyright (c) 2008 Opscode, Inc.
 # License:: Apache License, Version 2.0
 #
@@ -22,17 +23,21 @@ class Chef
   class Resource
     class Package < Chef::Resource
 
+      identity_attr :package_name
+
+      state_attrs :version, :options
+
       def initialize(name, run_context=nil)
         super
-        @resource_name = :package
-        @package_name = name
-        @version = nil
+        @action = :install
+        @allowed_actions.push(:install, :upgrade, :remove, :purge, :reconfig)
         @candidate_version = nil
+        @options = nil
+        @package_name = name
+        @resource_name = :package
         @response_file = nil
         @source = nil
-        @action = :install
-        @options = nil
-        @allowed_actions.push(:install, :upgrade, :remove, :purge, :reconfig)
+        @version = nil
       end
 
       def package_name(arg=nil)
@@ -74,7 +79,6 @@ class Chef
 	  :kind_of => [ String ]
 	)
       end
-
     end
   end
 end
