@@ -6,9 +6,9 @@
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
-# 
+#
 #     http://www.apache.org/licenses/LICENSE-2.0
-# 
+#
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS,
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -21,16 +21,18 @@ require 'chef/provider/package/smartos'
 
 class Chef
   class Resource
-    class SmartOSPackage < Chef::Resource::Package
-        
+    class SmartosPackage < Chef::Resource::Package
+
       def initialize(name, run_context=nil)
         super
         @resource_name = :smartos_package
         @provider = Chef::Provider::Package::SmartOS
       end
-      
+
     end
   end
 end
 
-
+# Backwards compatability
+# @todo remove in Chef 12
+Chef::Resource::SmartOSPackage = Chef::Resource::SmartosPackage
