@@ -18,6 +18,7 @@
 
 require 'chef/resource/apt_package'
 require 'chef/resource/bash'
+require 'chef/resource/batch'
 require 'chef/resource/breakpoint'
 require 'chef/resource/cookbook_file'
 require 'chef/resource/chef_gem'
@@ -33,6 +34,7 @@ require 'chef/resource/erl_call'
 require 'chef/resource/execute'
 require 'chef/resource/file'
 require 'chef/resource/freebsd_package'
+require 'chef/resource/ips_package'
 require 'chef/resource/gem_package'
 require 'chef/resource/git'
 require 'chef/resource/group'
@@ -48,10 +50,13 @@ require 'chef/resource/package'
 require 'chef/resource/pacman_package'
 require 'chef/resource/perl'
 require 'chef/resource/portage_package'
+require 'chef/resource/powershell_script'
 require 'chef/resource/python'
+require 'chef/resource/registry_key'
 require 'chef/resource/remote_directory'
 require 'chef/resource/remote_file'
 require 'chef/resource/rpm_package'
+require 'chef/resource/solaris_package'
 require 'chef/resource/route'
 require 'chef/resource/ruby'
 require 'chef/resource/ruby_block'
@@ -64,3 +69,5 @@ require 'chef/resource/template'
 require 'chef/resource/timestamped_deploy'
 require 'chef/resource/user'
 require 'chef/resource/yum_package'
+require 'chef/resource/lwrp_base'
+require 'chef/resource/bff_package'
